; ##############################################################################
; ##############################################################################
; ##############################################################################
;ADMUX  	ADC Multiplexer Selection Register									
; REFS1 REFS0 ADLAR  MUX3 MUX2 MUX1 MUX0 										
;REFS1 REFS0 Voltage Reference Selection										
; 0    0 AREF, Internal Vref turned off											
; 0    1 AVCC with external capacitor at AREF pin								
; 1    0 Reserved																
; 1    1 Internal 1.1V Voltage Reference with external capacitor at AREF pin	
; ADLAR: 		ADC Left Adjust Result											
; 																				
; MUX4:0: 	Analog Channel and Gain Selection Bits								
;  00000 	ADC0																
;  00001 	ADC1																
;  00010 	ADC2																
;  00011 	ADC3																
;  00100 	ADC4																
;  00101 	ADC5																
;  00110 	ADC6																
;  00111 	ADC7																
;  11110 	1.1V (VBG)															
;  11111 	 0 V (GND)															
;-------------------------------------------------------------------------------
;																					
; ADCSRA  ADC Control and Status Register A										
; ADEN: ADC Enable																	
; ADSC: ADC Start Conversion														
; ADATE: ADC Auto Trigger Enable													
; ADIF: ADC Interrupt Flag															
; ADIE: ADC Interrupt Enable														
; ADPS2:0: ADC Prescaler Select Bits												
;																					
; ADPS2 ADPS1 	ADPS0 Division Factor												
;  0 	0 		0 		  2															
;  0 	0 		1 		  2															
;  0 	1 		0 		  4															
;  0 	1 		1 		  8															
;  1 	0 		0 		 16															
;  1 	0 		1 		 32															
;  1 	1 		0 		 64															
;* 1 	1 		1 		128 (bei 22MHz)												
;																					
; ADCSRB  ADC Control and Status Register B										
; ACME: Analog Comparator Multiplexer Enable										
; ADTS2 ADTS1 ADTS0 	Trigger Source												
; *0 	0 		0 		Free Running mode											
;  0 	0 		1 		Analog Comparator											
;  0 	1 		0 		External Interrupt Request 0								
;  0 	1 		1 		Timer/Counter0 Compare Match								
;  1 	0 		0 		Timer/Counter0 Overflow										
;  1 	0 		1 		Timer/Counter1 Compare Match B								
;  1 	1 		0 		Timer/Counter1 Overflow										
;  1 	1 		1 		Timer/Counter1 Capture Event								
; ##############################################################################
; ##############################################################################
; ##############################################################################
ADC_INI_REF_1_1V:
	; REFERENZ  										

	ldi temp,(1<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(0<<MUX2)|(0<<MUX1)|(0<<MUX0) ; 1,1V			
	STS ADMUX, temp
	; "FREE Running" + Vorteiler (zw. 50kHz...200kHz)	
	ldi temp,((1<<ADEN)|(1<<ADSC)|(1<<ADATE)|(1<<ADIF)|(0<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)) 
	STS ADCSRA,temp       

	ldi temp,((0<<ACME)|(0<<ADTS2)|(0<<ADTS1)|(0<<ADTS0)) 
	STS ADCSRB,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ADC_INI_REF_5V:
	; REFERENZ  										
	ldi temp,(0<<REFS1)|(1<<REFS0)|(0<<ADLAR)|(0<<MUX3)|(0<<MUX2)|(0<<MUX1)|(0<<MUX0) ; AVCC + KAPAZITT	
	STS ADMUX, temp

	; "FREE Running" + Vorteiler (zw. 50kHz...200kHz)	
	ldi temp,((1<<ADEN)|(1<<ADSC)|(1<<ADATE)|(1<<ADIF)|(0<<ADIE)|(1<<ADPS2)|(1<<ADPS1)|(1<<ADPS0)) 
	STS ADCSRA,temp       

	ldi temp,((0<<ACME)|(0<<ADTS2)|(0<<ADTS1)|(0<<ADTS0)) 
	STS ADCSRB,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; U(VBG) x 1023 : ADC(VBG)														
ADC_MESSEN_BETRIEBSSPANNUNG_TEMP1:
.equ U_VBG									= 1100	; 1,1V	
	call ADC_MESSEN_VBG

	ldi temp1,LOW (U_VBG)
	ldi temp2,HIGH(U_VBG)
	ldi temp3,LOW (1023)
	ldi temp4,HIGH(1023)
	call MULTIPLIKATION_16_16_32

	LDS temp5,(adr_ADC_VBG_L)
	LDS temp6,(adr_ADC_VBG_H)
	call DIVISION_32_16

	ldi temp5,LOW (100)
	ldi temp6,HIGH(100)
	call DIVISION_32_16

	STS(adr_ADC_BETRIEBSSPANNUNG),temp1
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; MESSEN-VBG																	
ADC_MESSEN_VBG:
	LDS temp,ADMUX
	andi temp,0b11100000	; Bitmuster
	ori  temp,0b00011110	;  11110 	1.1V (VBG)
;	ori  temp,0b00011111	;  11111 	 0 V (GND)
	STS ADMUX,temp
	rcall ADC_TEMP1_2
	STS(adr_ADC_VBG_L),temp1
	STS(adr_ADC_VBG_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ADC_0_TEMP1_2:
	LDS temp,ADMUX
	andi temp,0b11100000	; Bitmuster
	STS ADMUX,temp
	rcall ADC_TEMP1_2
	STS(adr_ADC_0_L),temp1
	STS(adr_ADC_0_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ADC_1:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -1			; ADC1
	STS ADMUX,temp
	rcall ADC_TEMP1_2
	STS(adr_ADC_1_L),temp1
	STS(adr_ADC_1_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ADC_2:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -2			; ADC2
	STS ADMUX,temp
	rcall ADC_TEMP1_2
	STS(adr_ADC_2_L),temp1
	STS(adr_ADC_2_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ADC_3:
	LDS temp,ADMUX
	andi temp, 0b11100000
	subi temp, -3			; ADC3
	STS ADMUX,temp
	rcall ADC_TEMP1_2
	STS(adr_ADC_3_L),temp1
	STS(adr_ADC_3_H),temp2
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ADC-WERT 256 mal einlesen und daraus den Mittelwert bilden					
;																				
; OUT: 	TEMP1 (low)  															
;		TEMP2																	
; 																				
ADC_TEMP1_2:
	call wait_1ms		; ev. kleine PAUSE (fr Umschaltung)
	clr temp1			; HILFSREGISTER CLEAR							
	clr temp2
	clr temp3
	clr temp9			; SCHLEIFE initialisieren		
; neuen ADC-Wert lesen	 Wert verwerfen, da meist vom vorherigem ADC-Kanal	
ADC_DUMMY:
	LDS temp,(ADCSRA)	; auslesen		
	sbrs temp, ADIF		; skip if...	
	rjmp ADC_DUMMY
	ori temp,(1<<ADIF)
	STS(ADCSRA),temp	; ADIF is cleared by writing a logical one to the flag
; neuen ADC-Wert lesen	(Schleife - 256 mal)								
ADC_s:                
	LDS temp,(ADCSRA)	; auslesen		
	sbrs temp, ADIF		; skip if...	
	rjmp ADC_s
	ori temp,(1<<ADIF)
	STS(ADCSRA),temp	; ADIF is cleared by writing a logical one to the flag
	LDS  temp4,ADCL 		; immer zuerst LOW Byte lesen					
	LDS  temp5,ADCH 		; danach das mittlerweile gesperrte High Byte	
	tst temp4
	brne ADC_LED
	tst temp5
	breq ADC_w2
ADC_LED:
;-------------------------------------------------------------------------------
ADC_w2:
	cpi temp5,high(1023)
	brne ADC_MESSUNG_w
	cpi temp4,low (1023)
	brne ADC_MESSUNG_w
	LED_ROT_ON
;-------------------------------------------------------------------------------
; alle 256 ADC-Werte addieren												
ADC_MESSUNG_w:
	add temp1,temp4    	; addieren  						
	adc temp2,temp5    	; addieren ber Carry  				
	adc temp3,NULL    	; addieren ber Carry  				
	dec temp9			; Schleifenzhler MINUS 1			
	brne ADC_s			; ==> SCHLEIFE						
; MITTELWERT-BERECHNUNG	(128 addieren)										
	ldi temp,128
	add temp1,temp	     ; addieren  						
	adc temp2,NULL	     ; addieren ber Carry  				
	adc temp3,NULL    	 ; addieren ber Carry  				
; 	Ergebnis durch 256 teilen (temp5+6 nach temp1+2 kopieren)				
	mov temp1,temp2
	mov temp2,temp3
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################

